/*****************************************************************/
/*
/* browerlydanne.c copyright (c) 1994 Clemson University, all rights reserved.
/*
/* BROWERLYDANNE.C - Brouwer-Lydanne Orbital Model
/* This subroutine computes the osculating (actual) keplerian orbital
/* elements from Brouwer Mean elements.  The code generally follows
/* the nomenclature in "Mathematical Theory of the Goddard Trajectory
/* Determination System", X-582-76-77.
/* 
/* Calling Sequence:
/* initbrowerlydanne(brw)  initialize brouwer elements
/* browerlydanne(del, osc) return osculating elements
/* 
/* DEL  - R*8 Delta Time From Brouwer elements to time of prediction. (sec)
/* BRW  - R*8 Brouwer Mean Orbital Elements Array:
/* BRW (1) Semi-Major Axis 	(km )
/* BRW (2) Eccentricity    	(   )
/* BRW (3) Inclination     	(rad)
/* BRW (4) Ascending Node  	(rad)
/* BRW (5) Angle of Perigee	(rad)
/* BRW (6) Mean Anomaly    	(rad)
/* 
/* OSC  - R*8 Brouwer Mean Orbital Elements Array
/* OSC (1) Semi-Major Axis 	(m  )
/* OSC (2) Eccentricity    	(   )
/* OSC (3) Inclination     	(rad)
/* OSC (4) Ascending Node  	(rad)
/* OSC (5) Angle of Perigee	(rad)
/* OSC (6) Mean Anomaly (rad)
/* OSC (7)  Mean Motion(rad/sec)
/* OSC (8)  Perigee motion(rad/sec)
/* OSC (9)  ascending node rate (rad/sec)
/* 
/* IER  - I*2 completion Code 1 = success 2 = invalid input parameter
/*
/* $Log: browerlydanne.c,v $
 * Revision 1.2  1995/03/10  16:22:05  rheil
 * Added math library to insure proper function returns
 *
 * Revision 1.1  1995/02/28  17:12:54  rheil
 * Initial revision
 *
/*  10/28/1987  - Coded by C. Hoisington  SSAI Version 1.0
/*  11/27/1994  - Code Modification, Robert M. Heil (Bob)
/*
/******************************************************************/
/* UNIX INCLUDE FILES */
#include <stdio.h>
#include <math.h>
 
/* CREATE_1B INCLUDE FILES */
#include "macros.h"
#include "types.h"
#include "hrpt.h"

/* CONSTANTS */
static flt8 c1 = 6.2831853071795862;
static flt8 u  = 3.986031995e14;
static flt8 k2 = 22021061317.857616;
static flt8 k3 = 657981805781804.25;
static flt8 k4 = 1.0082337338923276e21;
static flt8 k5 = 2.3959206033836406e27;

/* GLOBAL VARIABLES */
static char browerlydanne_c[] = "$Header: /projects/level1b/create.1bl/C/RCS/browerlydanne.c,v 1.2 1995/03/10 16:22:05 rheil Exp $";
static flt8 axis, ecc, ascn, w, m, temp, n, nhalf, n3half, n2, n3, q, q2;
static flt8 q4, c1mq2, c3q2m1, g2, g3 , g4, g5, gp2, gp3, gp4, gp5, ic;
static flt8 ecc2, sipp, sippd2, cippd2, tippd2;
static flt8 ap1, ap2, a1, a2, a3, a4, a5, a6, a7, a8, ap8, a9, a10, a11;
static flt8 a12, a13, a14, a15, a16, a17, a18, a20, a21, a25, a26, b1;
static flt8 b2, b3, b4, b5, b6, b7, b8, b9, b10, b11, b12, b13, b14, b15;
static flt8 ld, gd, hd, md;

/* FUNCTIONS */
/* flt8 d_mod(flt8, flt8)*/
/* void initbrowerlydanne(flt4)  */
/* void browerlydanne(flt8, flt8) */

/* Builtin functions */
flt8 sqrt(), cos(), sin(), tan(), d_mod(), atan2(), asin();
/******************************************************************/

flt8 d_mod(x,y)
flt8 *x, *y;
{
     flt8 floor(), quotient;
     if( (quotient = *x / *y) >= 0)
          quotient = floor(quotient);
     else
          quotient = -floor(-quotient);
     return(*x - (*y) * quotient );
}

void initbrowerlydanne(brw)
flt4 *brw;
{
    flt8 incl, axis2, n0, q3; 

    /* q3 NOT DEFINED OR EVER USED? line 226*/

    int ier = 1;   /* assume elements ok */
    /* ier was a parameter but was not used, could easily be put back */

/*  Load Brouwer elements and reduce to prin angles and make error checks. */
/*  pg. 5-43 eq. 5-144a-c */

    axis = brw[0];     /* Semi-Major axis */
    ecc  = brw[1];     /* Eccentricity */
    incl = brw[2];     /* Inclination */
    ascn = brw[3];     /* Ascending node */
    w    = brw[4];     /* Angle of perigee */
    m    = brw[5];     /* Mean anomaly */

    if (axis < 6378149.196)     /* semi-major axis negat. */
       ier = 2;
    if (ecc < 0 || ecc > 1)/* invalid eccentricity */
       ier = 2;
    if (incl < 0 || incl > c1) 
       ier = 2;
    if (ascn < 0 || ascn > c1) 
       ier = 2;
    if (w < 0 || w > c1) 
       ier = 2;
    if (m < 0 || m > c1) 
       ier = 2;

/*  Compute some intermediate variables */
/*  These expression will be used repeatedly in following computations */
/*  pg 5-44,45 */
    axis2 = axis*axis;         /* axis**2 */
    temp = axis*axis*axis;    /* axis**3 */
    n0 = sqrt(u / (temp));
    n = 1 - ecc * ecc;
    nhalf = sqrt(n);
    n3half = n * nhalf;        /* N**3/2 */
    n2 = n * n;                /* N**2 */
    n3 = n * n2;               /* N**3 */
    q = cos(incl);             /* theta=cos(i) */
    q2 = q * q;                /* Q**2 */
    q4 = q2 * q2;              /* Q**4 */
    c1mq2 = 1 - q2;            /* (1-Q**2) */
    c3q2m1 = q2 * 3 - 1;       /* (3*Q**2-1) */
    g2 = k2 / axis2;           /* gamma2=k2/a"**2 */
    g3 = k3 / temp;           /* gamma3=k3/a"**3 */
    g4 = k4 / (axis2 * axis2); /* gamma4=k4/a"**4 */
    g5 = k5 / (axis2 * temp); /* gamma5=k5/a"**5 */

    gp2 = g2 / n2;             /* gamma'2=G2/n**4 */
    gp3 = g3 / n3;             /* gamma'3=G3/n**6 */
    gp4 = g4 / (n2 * n2);      /* gamma'4=G4/n**8 */
    gp5 = g5 / (n2 * n3);      /* gamma'5=G5/n**10 */
    ecc2 = ecc * ecc;          /* e"**2 */
    sipp = sin(incl);          /* sine (i") */
    sippd2 = sin(incl / 2);    /* sine (i"/2) */
    cippd2 = cos(incl / 2);    /* cos  (i"/2) */
    tippd2 = tan(incl / 2);    /* tan  (i"/2) */

/*  IC=0 not critical IC=1 critical angle */
    ic = 0;
    temp = 1 / (1 - q2 * 5);
    if (q4 * 25 * q * g2 * ecc2 * temp * temp > 1)
       ic = 1;

/*  SEC5 - "A" Intermediate Variables   pg. 5-55 eq. 5-184 */
    if (ic == 0)      /* skip if ic critical */
    {
        ap1  = temp;
        ap2  = q2 * 3 + q4 * 8 * ap1;
        a1   = gp2 * 0.125 * n * (1 - q2 * 11 - q4 * 40 * ap1);
        a2   = gp4 * 0.41666666666666669 / gp2 * n * (1 - ap2);
        a3   = gp5 / gp2 * (ecc2 * 3 + 4);
        a4   = gp5 / gp2 * (1 - ap2 * 3);
        a5   = a3 * (1 - ap2 * 3);
        a6   = gp3 * 0.25 / gp2;
        a7   = a6 * n * sipp;
        temp = gp5 / gp2 * (1 - q2 * 5 - q4 * 16 * ap1);
        a8   = temp * ecc2;
        ap8  = temp * ecc;
        a9   = n * sipp;
        a10  = ecc2 + 2;
        a11  = ecc2 * 3 + 2;
        a12  = a11 * q2;
        a13  = (ecc2 * 5 + 2) * q4 * ap1;
        temp = q, temp *= q, temp *= temp;
        a14  = ecc2 * q * temp * ap1 * ap1;
        a15  = q2 * ap1;
        a16  = a15 * a15;
        a17  = ecc * sipp;
        a18  = a17 / (nhalf + 1);
        a20  = ecc * q;
        a21  = ecc * a20;
        a25  = a15 * 16 + a16 * 40 + 3;
        a26  = a21 * 0.125 * (a16 * 200 + 11 + a15 * 80);

    /*  SEC6 - "B" Intermediate Variables   pg. 5-56 eq. 5-185 */
        b1   = nhalf * (a1 - a2) - ((a10 - a14 * 400 - a13 * 40 - a12 * 11)/16 +
               (a16 * 200 + 11 + a15 * 80) * a21 / 8) * gp2 + (a14 * -80 - 
               a13 * 8 - a12 * 3 + a25 * 2 * a21 + a10) * 5 / 24 * gp4 / gp2;
        b2   = a6 * a18 * (n + 2 - ecc2) + a5 * 0.078125 * a18 * n - a4 * 
               0.46875 * a17 * n3half + a20 * tippd2 * (a5 * 0.078125 + a6) + 
               a4 * 0.078125 * a17 * (ecc2 * 9 + 26) + a3 * 0.46875 * a20 * 
               a25 * sipp * (1 - q);
        b3   = gp5 * 0.060763888888888888 / gp2 * ecc * sipp * (q - 1) * a21 * 
               (a16 * 80 + 5 + a15 * 32) - ap8 * 0.030381944444444444 * (a21 *
               tippd2 + (ecc2 * 2 + (1 - n3half) * 3) * sipp);
        b4   = nhalf * ecc * (a1 - a2);
        b5   = nhalf * (a4 * 0.078125 * a9 * (ecc2 * 9 + 4) + a7);
        b6   = n3half * .091145833333333329 * a8 * sipp;
        b7   = n * a17 * ap1 * (gp2 * 0.125 * (1 - q2 * 15) - gp4 * 
               0.41666666666666669 / gp2 * (1 - q2 * 7));
        b8   = a3 * 0.078125 * n * (1 - q2 * 9 - q4 * 24 * ap1) + n * a6;
        b9   = n * 0.091145833333333329 * a8;
        b10  = sipp * (gp4 * 0.41666666666666669 / gp2 * a21 * a25 - a26 * gp2);
        b11  = a21 * (a5 * 0.078125 + a6 + a3 * 0.46875 * a25 * sipp * sipp);
        b12  = -((a16 * 80 + a15 * 32 + 5) * (gp5 * 0.060763888888888888 / gp2 *
               ecc * sipp * sipp * a21) + a8 * 0.030381944444444444 * a20);
        b13  = ecc * (a1 - a2);
        b14  = a5 * 0.078125 * n * sipp + a7;
        b15  = a8 * 0.091145833333333329 * n * sipp;
    }

/*  SEC7 Compute Secular Anguler Velocities and Displacements */
/*       Mdot is the mean motion, Gdot is the perigee motion */
/*       Hdot is the ascending node motion  pg. 5-44 eq. 5-146-148 */
    ld = n0 * nhalf * (gp2 * (c3q2m1 * 1.5 + gp2 * 0.09375 * (n * 25 + nhalf * 
         16 - 15 + (30 - nhalf * 96 - n * 90) * q2 + (nhalf * 144 + 105 + n * 
         25) * q4)) + gp4 * 0.9375 * ecc2 * (3 - q2 * 30 + q4 * 35));
    gd = n0 * (gp2 * ((q2 * 5 - 1) * 1.5 + gp2 * 3 / 32 * (n * 25 + nhalf * 
         24 - 35 + (90 - nhalf * 192 - n * 126) * q2 + (nhalf * 360 + 385 + n *
         45) * q4)) + gp4 * 0.3125 * (21 - n * 9 + (n * 126 - 270) * 
         q2 + (385 - n * 189) * q4));
    hd = n0 * (gp2 * (gp2 * 0.375 * ((n * 9 + nhalf * 12 - 5) * q - (nhalf * 36+
         35 + n * 5) * q3) - q * 3) + gp4 * 1.25 * q * (5 - n * 3) * 
         (3 - q2 * 7));
    md = ld + n0;
} /* initbrowerlydanne */

/*****************************************************************************/
void browerlydanne (del, osc)
flt8 del, *osc;
{
    flt8 lpp, gpp, hpp, x2gpp, ecx, ecca, cecc, fpp, sfpp, cfpp;
    flt8 x2fpp, arpp, arpp2, di, sidh, il, edl, d1e, de, e, l;
    flt8 t1, t2, a, h, lghp, lgh, g;
    int i;

/* Secular Displacements pg. 5-43 eq 5-144d,f */
    temp = m + del * md;
    lpp = d_mod(&temp, &c1);     /* l" mean anomaly */
    temp = w + del * gd;
    gpp = d_mod(&temp, &c1);     /* g" perigee */
    temp = ascn + del * hd;
    hpp = d_mod(&temp, &c1);     /* h" ascending node */
    if (lpp < 0)
       lpp += c1;
    if (gpp < 0)
       gpp += c1;
    if (hpp < 0)
       hpp += c1;
    x2gpp = gpp * 2;            /* 2*GPP */

/*  SEC8 - Compute Eccentric Anomaly
           Eccentric anomaly, E" is computed iteratively from Kepler's
           equation  E"-e"sin(E")=l"  pg. 5-58 eq. 5-186 */
    ecx = 0;
    ecca = lpp; /* initialize Ecc. Anom. */
    for (i = 1; i <= 12; ++i) 
    {
        ecca = lpp + ecc * sin(ecca);
    /*  Kepler's equation */
        if ((temp = ecca - ecx, abs(temp)) < 1e15)
           i = 13;
        else
           ecx = ecca;
    }

/*  SEC9 -  Compute True anomaly  pg. 5-58 eq. 5-187 */
    cecc = cos(ecca);                           /* cos (E") */
    fpp = atan2(nhalf * sin(ecca), cecc - ecc); /* f" */
    fpp = d_mod(&fpp, &c1);
    if (fpp < 0)
       fpp += c1;
    sfpp = sin(fpp);                            /* sine (f") */
    cfpp = cos(fpp);                            /* cos  (f") */
    x2fpp = fpp * 2;                            /* 2*f" */

/*  SEC10 - Compute mean radial distance pg. 5-58 eq. 5-188,189 */
    arpp = 1 / (1 - ecc * cecc);                /* a"/r" */
    arpp2 = arpp * arpp;                        /* (a"/r")**2 */

/*  SEC11 -  Compute orbital inclination di is the long + short period incl*/
    di = sidh = 0;
    if (ic == 0) /* skip if critical */
    {
    /* del i pg. 5-53 eq. 5-176   pg. 5-53 eq. 5-175 */
       di = q * 0.5 * gp2 * sipp * (ecc * cos(fpp * 3 + x2gpp) + (ecc * 
            cos(x2gpp + fpp) + cos(x2fpp + x2gpp)) * 3) - a20 / n * (b7 * 
            cos(x2gpp) + b8 * sin(gpp) - b9 * sin(gpp * 3));

    /* sin (i"/2)dh pg. 5-53 eq. 5-174 */
       sidh = 1 / cippd2 * ((b10 * sin(x2gpp) + b11 * cos(gpp) + b12 * 
              cos(gpp * 3) - gp2 * q * sipp * ((ecc * sfpp - lpp + fpp) * 6 - 
              ((sin(x2gpp + x2fpp) + ecc * sin(x2gpp + fpp)) * 3 + ecc * 
              sin(fpp * 3 + x2gpp))) * 0.5) * 0.5);
    }
/*  Orbital incl il computing 2nd power */
    temp = di * 0.5 * cippd2 + sippd2;
    il = asin((sqrt(sidh * sidh + temp * temp))) * 2;
    il = d_mod(&il, &c1);
    if (il < 0)
       il += c1;

/*  SEC12 - Compute e"dl, d1e
            e"dl is the long period mean anomaly terms
            d1e  is the long period eccentricity terms  pg. 5-53 eq. 5-172+3 */
    edl = d1e = 0;
    if (ic == 0)   /* skip if critical */
    {
       edl = b4 * sin(x2gpp) - b5 * cos(gpp) + b6 * cos(gpp * 3) - n3half * 
             0.25 * gp2 * (c3q2m1 * 2 * (n * arpp2 + arpp + 1) * sfpp + c1mq2 *
             3 * ((-n * arpp2 - arpp + 1) * sin(x2gpp + fpp) + (n * arpp2 + 
             arpp + 0.33333333333333331) * sin(fpp * 3 + x2gpp)));
       d1e = b13 * cos(x2gpp) + b14 * sin(gpp) - b15 * sin(gpp * 3);
    }

/*  SEC14 -  Compute de - short period eccentricty terms pg. 5-52 eq. 5-171 */
/* Computing 2nd power */
    temp = cfpp * cfpp;
    de = d1e - n / 2 * (gp2 * c1mq2 * (cos(x2gpp + fpp) * 3 + cos(fpp * 3 + 
         x2gpp)) - g2 * 3 / n3 * c1mq2 * cos(x2gpp + x2fpp) * (ecc * 3 * 
         temp + cfpp * 3 + ecc2 * cfpp * temp + ecc) - g2 / n3 * c3q2m1 * 
         (ecc * nhalf + ecc / (nhalf + 1) + ecc * 3 * temp + cfpp * 3 + ecc2 * 
         cfpp * temp));

/*  SEC15 - Compute eccentricity pg. 5-52 eq. 5-170 */
    e = sqrt((ecc+de) * (ecc+de) + edl * edl);

/*  SEC16 - Compute Mean Anomaly pg. 5-54 eq. 5-177 */
    l = 0;
    if (e != 0) 
    {
       t1 = edl * cos(lpp) + (ecc + de) * sin(lpp);
       t2 = (ecc + de) * cos(lpp) - edl * sin(lpp);
       l = atan2(t1, t2);
       l = d_mod(&l, &c1);
       if (l < 0)
          l += c1;
    }

/*  SEC17 -  Compute Semi-Major Axis pg. 5-52 eq. 5-169 */
    a = axis * (g2 * (c3q2m1 * ecc / n3 * (ecc * nhalf + ecc / (nhalf + 1) + 
        cfpp * (ecc * 3 * cfpp + 3 + ecc2 * cfpp * cfpp)) + c1mq2 * 3 * 
        arpp * arpp * arpp * cos(x2fpp + x2gpp)) + 1);

/*  SEC18 - Compute Ascending Node  pg. 5-54 eq. 5-179 */
    h = 0;
    if (il != 0) 
    {
       t1 = sidh * cos(hpp) + sin(hpp) * (di * 0.5 * cippd2 + sippd2);
       t2 = cos(hpp) * (di * 0.5 * cippd2 + sippd2) - sidh * sin(hpp);
       h = atan2(t1, t2);
       h = d_mod(&h, &c1);
       if (h < 0)
          h += c1;
    }

/*  SEC19 - Compute Argument of Perigee   g = (l+g+h)-l-h */
    lghp = 0;
    if (ic == 0)  /* skip if critical */
    {
       lghp = lpp + gpp + hpp + b3*cos(gpp*3) + b1 * sin(x2gpp) + b2 * cos(gpp);
       lghp = d_mod(&lghp, &c1);
       if (lghp < 0)
          lghp += c1;
    }
    lgh = lghp + n * 0.25 / (nhalf + 1) * ecc * gp2 * (c1mq2 * 3 * (sin(fpp * 
          3 + x2gpp) * (arpp2 * n + 0.33333333333333331 + arpp) + sin(x2gpp + 
          fpp) * (1 - arpp2 * n - arpp)) + sfpp * 2 * c3q2m1 * (arpp2 * n + 1 +
          arpp)) + gp2 * 1.5 * ((q2 * 5 - q * 2 - 1) * (ecc * sfpp + fpp - 
          lpp)) + (q * 2 + 3 - q2 * 5) * (gp2 * 0.25 * (ecc * sin(fpp * 3 + 
          x2gpp) + (sin(x2gpp + x2fpp) + ecc * sin(x2gpp + fpp)) * 3));
    lgh = d_mod(&lgh, &c1);
    if (lgh < 0)
       lgh += c1;
    temp = lgh - l - h;
    g = d_mod(&temp, &c1);
    if (g < 0)
       g += c1;

/*  SEC20 - Compute final elements */
    osc[0] = a;       /* Semi-major axis */
    osc[1] = e;       /* eccentricity */
    osc[2] = il;      /* inclination */
    osc[3] = h;       /* ascending node h */
    osc[4] = g;       /* angle of perigee */
    osc[5] = l;       /* mean anomaly */
    osc[6] = md;      /* mean motion */
    osc[7] = gd;      /* perigee dot */
    osc[8] = hd;      /* ascending node dot */
} /* browerlydanne */
