/*****************************************************************/
/*
/* earthlocate.c copyright (c) 1994 Clemson University, all rights reserved.
/*
/* EARTHLOCATE.C - EARTH LOCATION ROUTINE FOR PASS SCHEDULING
/*      The earth location subroutine gives the location in degrees pf   
/*      Latitude And Longitude for the Specifed Scan Angles.   
/*      The Locations Are Computed Using The Date And Time Of The   
/*      Scan  And The Orbital  (Brouwer Mean) Elements,   
/*
/* ARGUMENTS:   
/* SIGV (INPUT)  - Array That Holds The Specified Scan   
/*                 Angles (Units: Degrees) ... Includes 0.0 Deg.   
/* LON(OUTPUT)   - Array That Gives The Position Of The Pixel   
/*                 In Degrees Longitude For Each Scan Angle.   
/*                 [Range: -180 Deg. To +180 Deg.]   
/* LAT(OUTPUT)   - Array That Gives The Position Of The Pixel   
/*                 In Degrees Latitude For Each Scan Angle.   
/*                 [Range: -90 Deg. To +90 Deg.]   
/* 
/* The Following Variables are defined as:
/* DTUT  : Astronomical Time Correction. Defined As UT1-UTC . Available 
/*         from U.S Naval Observatory(Washington D.C.). In this program 
/*         it is set to 0. 
/* DP    : Julian Days Since Jan 1, 1900 To Jan 1 of Present Year. 
/* PHIGO : Right Ascension Of Greenwich With Respect To Aries At Jan 0.0
/*
/* $Log: earthlocate.c,v $
 * Revision 1.1  1995/02/28  17:12:54  rheil
 * Initial revision
 *
/*  09/18/91  - R.Mahoney/SSAI modified for pass scheduler   
/*  10/01/94  - C Code Modification, Robert M. Heil (Bob)
/*  01/10/95 - Support SPARC as well as ALPHA, can run multiple processors.
/*
/******************************************************************/
/* UNIX INCLUDE FILES */
#include <stdio.h>
 
/* CREATE_1B INCLUDE FILES */
#include "macros.h"
#include "types.h"
#include "hrpt.h"

/* GLOBAL VARIABLES */
static char earthlocate_c[] = "$Header: /projects/level1b/create.1bl/C/RCS/earthlocate.c,v 1.1 1995/02/28 17:12:54 rheil Exp $";

/* FUNCTIONS */
/* void earthlocate(flt8, flt8, int4, flt8, flt8, flt8)
/******************************************************************/

void earthlocate(phig, vke, nsig, sigv, lat, lon)
flt8 phig, *vke;
int4 nsig;
flt8 *sigv, *lat, *lon;
{
    static flt8 alp  = 0.0, bet  = 0.0, gam  = 0.0;
    static flt8 alpp = 0.0, betp = 0.0, gamp = 0.0, alps, bets, gams;
    static flt8 aa   = 6378.155;
    static flt8 e2   = 0.0066944614;
    static flt8 dr, e12, eps, axis, ecc, incl, lont, w, m, si, ci, rp;
    static flt8 f, sisf, sig, ssig, csig, amux, amuz, c1, c2, c3;
    static flt8 ac, bc, cc, xa, ya, za, rho, anu, p, q, sp, cp, sq, cq;
    static flt8 a1, b1, xr, yr, zr, rhop;

    int4 i;
    flt8 sin(), cos(), sqrt(), atan2();


/*  MULTIPLIER TO CONVERT RADIANS TO DEGREES: */
    dr = 57.29577951308238;
    e12 = 1 - e2;
    eps = e2 / e12;

/*  SET THE INSTANTANEOUS ARRAY ELEMENTS = INDIVIDUAL VARIABLES: */
    axis = vke[0]/1e3; /*  SEMI-MAJOR AXIS      [Kilometers] */
    ecc  = vke[1];     /*  ECCENTRICITY         [Dimensionless] */
    incl = vke[2];     /*  INCLINATION          [radians] */
    lont = vke[3];     /*  LONGITUDE            [Radians] */
    w    = vke[4];     /*  ARGUMENT OF PERIGEE  [Radians] */
    m    = vke[5];     /*  MEAN ANOMALY         [Radians] */
    si = sin(incl);
    ci = cos(incl);

/*  CALCULATE RP, THE DISTANCE FROM EARTH'S CENTER TO SATLLITE: */
    rp = axis * (1 - ecc * ecc) / (ecc * cos(m) + 1);

/*  COMPUTE ANGULAR DISPLACEMENT "F" FROM THE EQUATOR ALONG SATELLITE */
/*  ORBIT AT TIME = T  BASED ON OSCULATING ELEMENTS. */
    f = w + m + ecc * 2 * sin(m);
    sisf = si * sin(f);

/*  CONVERT SPACECRAFT & INSTRUMENT ANGLES FROM DEGREES TO RADIANS: */
    alp /= dr;   /* ROLL */
    bet /= dr;   /* PITCH */
    gam /= dr;   /* YAW */
    alpp /= dr;
    betp /= dr;
    gamp /= dr;

/*  INSTRUMENT OFFSET ANGLES */
    alps = alp + alpp;
    bets = bet + betp;
    gams = gam + gamp;

/*  SOLVE GEOMETRY FOR EACH SCAN ANGLE */
/*  (Intersection of Scan Vectors with Oblate Spheroidal Earth) */
    for (i = 0; i < nsig; ++i) 
    {
        sig = sigv[i] / dr;
        ssig = sin(sig);
        csig = cos(sig);

    /*  QUADRATIC COEFFICIENTS */
        amux = -(gams * ssig + bets * csig) / (alps * ssig - csig);
        amuz = (ssig + alps * csig) / (alps * ssig - csig);
        c1 = eps * sisf * sisf + 1;
        c2 = (eps * ci * ci + 1) * amuz * amuz;
        c3 = eps * sisf * ci * amuz;
        ac = c1 + c2 + c3 * 2;
        bc = rp * (-2) * (c2 + c3);
        cc = rp * rp * c2 - aa * aa;

    /*  INTERSECTION COORDINATE IN AUX. REFERENCE FRAME */
        ya = (-bc + sqrt(bc * bc - ac * 4 * cc)) / (ac * 2);
        xa = amux * (ya - rp);
        za = amuz * (ya - rp);

    /*  TRANSFORM COORDINATES TO ROTATING EARTH FRAME */
        rho = sqrt(xa * xa + ya * ya);
        anu = atan2(ya, xa);
        p = f + anu;
        q = lont - phig;
        sp = sin(p);
        cp = cos(p);
        sq = sin(q);
        cq = cos(q);
        a1 = rho * ci * cp + si * za;
        b1 = rho * sp;
        xr = a1 * sq + b1 * cq;
        yr = b1 * sq - a1 * cq;
        zr = -rho * si * cp + ci * za;
        rhop = sqrt(xr * xr + yr * yr);

        lon[i] = atan2(yr, xr) * dr;         /* LONGITUDE */
        lat[i] = atan2(zr, rhop * e12) * dr; /* LATITUDE  */
    }
}
