/*****************************************************************/
/*
/* headers.c copyright (c) 1994 Clemson University, all rights reserved.
/*
/* HEADERS.C - These two routines construct the minor frame headers
/*             for the 1B output file. 
/*
/* GEN_TBM - Routine to construct 122 byte TBM Header
/* 
/* Inputs :
/*   Structure runrec: 
/*      startlat      > double  Starting Latitude
/*      startlon      > double  Starting Longitude
/*      endlat        > double  Ending   Lattiude
/*      endlon        > double  Ending   Longitude
/*      starttime     > int     Starting Time
/*      startdate     > int     Starting Date
/*      endtime       > int     Ending   Time
/*      enddate       > int     Ending   Date
/* 
/*   Structure hrpt_header:
/*      orbit_number  > int     orbit number
/*      spacecraft_id > int     common satellite id  6,7,8,9,10,11,12....
/*      pass_year     > int     
/* 
/*   Structure rmc:
/*      cnoaaid       > char[2] noaa satellite letter id (NG, NC, NH etc)
/* 
/* Outputs :
/*      tbm           > char[122] TBM header
/*
/* GEN_HEADER - SUBROUTINE TO CREATE IB TAPE DATA HEADER
/* 
/* DATA TYPE CODE   bits 1-4:   1 = LAC         6 = MSU
/*                              2 = GAC         7 = SSU
/*                              3 = HRPT        8 = DCS
/*                              4 = TIP         9 = SEM
/*                              5 = HIRS/2   10-15= SPARE
/* 
/*                  bits 5-8:   1 = EMBEDDED  TIP
/*                              2 = STORED    TIP
/*                              3 = THIRD CDA TIP
/*                              4-15 = SPARE
/* 
/* Inputs :
/*   Structure hrpt_header:
/*      spacecraft_id > int     common satellite id  6,7,8,9,10,11,12....
/* 
/*   Structure runrec: 
/*      startdate     > int     Starting Date
/*      enddate       > int     Ending   Date
/*  
/* Outputs :
/*      data_hdr   > int2[7400] data set header
/*
/* $Log: headers.c,v $
 * Revision 1.2  1995/06/07  13:31:30  rheil
 * Semi Stable version for alpha, sparc, hp, and pentium.  Multiple processors.
 *
 * Revision 1.1  1995/02/28  17:12:54  rheil
 * Initial revision
 *
/*  12/13/91 - R. Mahoney/SSAI - new version
/*  12/23/91 -                   screen management display
/*  10/11/94 - Code Modification, Robert M. Heil (Bob)
/*
/******************************************************************/
/* UNIX INCLUDE FILES */
#include <stdio.h>
 
/* CREATE_1B INCLUDE FILES */
#include "macros.h"
#include "types.h"
#include "hrpt.h"

/* GLOBAL VARIABLES */
static char headers_c[] = "$Header: /projects/level1b/create.1bl/C/RCS/headers.c,v 1.2 1995/06/07 13:31:30 rheil Exp $";
/******************************************************************/
 
void gen_tbm(header,rundata,cal,tbm)
hrpt_header *header;
runrec      *rundata;
rmc         *cal; 
byte        *tbm;
{
    flt4 stime, etime;
    int4 nmin, onum;

    /* SITE LOCATION AND DATA TYPE:  TBM(31:38 ) = 'SPR.HRPT'
       NOAA SPACE CRAFT ID:          TBM(39:42) ='.'//CNOAAID(1:2)//'.'
       YEAR AND JULIAN DAY:
       ENCODE(2,100,CYR)Pass_yr      TBM(43:45) = 'D'//CYR(1:2)
       ENCODE(3,101,CDAY)Start_date  TBM(46:49) = CDAY(1:3)//'.' */
    if(rundata->d_flag)
       trace("building tbm header ... ");
    sprintf(&tbm[30],"SPR.HRPT.%.2s.D%2d%03d.",cal->cnoaaid,
             header->pass_year,rundata->startdate);

    /* START and END TIME in HOURs AND MINUTES */
    stime = rundata->starttime/3600000.0;
    etime = rundata->endtime/3600000.0;
    if ((nmin  = (rundata->endtime - rundata->starttime)/60000) < 0)
       nmin  = (rundata->endtime + 86400 - rundata->starttime)/60000;
    sprintf(&tbm[49],"S%02d%02d.E%02d%02d.",(int) stime,(int) ((stime - 
       (int) stime) * 60.0),(int) etime,(int) ((etime - (int)etime) * 60.0));

    /* ORBIT NUMBER: TBM(62:62)='B'          TBM(63:67) = corbn(1:5)
                     TBM(68:69) = corbn(4:5) TBM(70:70)='.'
       HRPT INGEST STATION LOC:              TBM(71:74)='BA  '  !Bangladesh */
    onum = header->orbit_number;
    sprintf(&tbm[61],"B%05d%02d.BA  ", onum, onum - (int)(onum / 100) * 100);

    /* START & END LAT LON: TBM(75:75) ='S'          TBM(86:89) = CELON(1:4) 
                            TBM(76:78) = CSLAT(1:3)  TBM(90:91) = CSHR(1:2)
                            TBM(79:81) = CELAT(1:3)  TBM(92:93) = CSMN(1:2) 
                            TBM(82:85) = CSLON(1:4)  TBM(94:96) = CNMIN(1:3) */
    sprintf(&tbm[74],"S%+03d%+03d%+04d%+04d%.4s%03d", 
           (int) rundata->startlat, (int) rundata->endlat,
           (int) rundata->startlon,(int) rundata->endlon,&tbm[50],nmin);

    /* CHANNEL SELECTION FLAGS: TBM(97:117) = 'N11111000000000000000'  */
    sprintf(&tbm[96],"N11111000000000000000");
    if(rundata->d_flag)
       trace("done.\n");
}

void gen_header(header,rundata,data_hdr)
hrpt_header *header;
runrec      *rundata;
int2        *data_hdr;
{
    if(rundata->d_flag)
       trace("building first data set header ... ");
    /* SPACECRAFT ID CODE and DATA TYPE: */
    /* CALL MVBITS(ID,0,8,HDR(1),0) */
    /* CALL MVBITS(1,0,4,HDR(1),8) */
    /* CALL MVBITS(3,0,4,HDR(1),12) */
    data_hdr[0] = (header->spacecraft_id & 0xff);
    data_hdr[0] = ((3 & 0xf) << 12)|((1 & 0xf) <<8)|
                             (header->spacecraft_id & 0xff);

    /* START TIME: year and day */
    /* KSDAY=ISDAT */
    /* CALL MVBITS(IYR ,0,7,HDR(2),1) */
    /* CALL MVBITS(JSDAY(1),8,1,HDR(2),0) */
    /* CALL MVBITS(JSDAY(1),0,8,HDR(2),8) */
    data_hdr[1] = ((header->pass_year & 0x7f) << 1) | 
       ((rundata->startdate & 0xff) << 8) | ((rundata->startdate & 0x100) >> 8);

    /* NUMBER OF SCANS  HDR(5)=NSCN     CALL GEN_FOR_BSWP2( hdr(5) )  */
    data_hdr[4] = ((rundata->nscans >> 8) & 0xff) | 
                          ((rundata->nscans & 0xff) << 8);

    /* END TIME: year and day */
    /* KEDAY=IEDAT */
    /* CALL MVBITS(  IYR,0,7,HDR(6),1)    !END TIME */
    /* CALL MVBITS( JEDAY(1),8,1,HDR(6),0) */
    /* CALL MVBITS( JEDAY(1),0,8,HDR(6),8) */
    data_hdr[5] = ((header->pass_year & 0x7f) << 1) | 
        ((rundata->enddate & 0xff) << 8) | ((rundata->enddate & 0x100) >> 8);

    /* NUMBER OF DATA GAPS: HDR(13)=NGAP CALL GEN_FOR_BSWP2( HDR(13)) */
    /* The ngaps is never calculated correctly? Filling in with zero now */
    /*
    data_hdr[12] = ((rundata->ngaps >> 8) & 0xff) | 
                           ((rundata->ngaps & 0xff) << 8);
    */
    data_hdr[12] = -1;
    if(rundata->d_flag)
       trace("done.\n");
}
