/*****************************************************************/
/*
/* zenith.c copyright (c) 1994 Clemson University, all rights reserved.
/*
/* ZENITH.C - This routine calculates the solar zenith angle at LEN points    
/*    along a particular scan.   The routine 'Ertloc' is called to    
/*    Determine The Position Of Each Point (Pixel).                   
/*                                                                    
/*    After determining the location, 'Zenith' reads the date and     
/*    time from ibuf(), which is passed to 'Zenith' from the main     
/*    program .  From The Location And Time, 'Zenith'  computes       
/*    the hour angle (H).  The date and position are used to          
/*    compute the solar inclination angle (or solar declination).     
/*    This is done using the assumption that this time function       
/*    behaves as a sine wave (Kuo-nan Liou, P.47) with delta=0 degrees
/*    at the vernal and autumnal equinoxes.  begin the annual sine    
/*    wave on the vernal equinox. (notation Also Similar To Liou)     
/*                                                                    
/*    ARGUMENTS:                                                      
/*                                                                    
/*    ZEN....(OUTPUT)  Vector That Contains The LEN Zenith Angles     
/*                     Calculated By This Subroutine For The Scan.     
/* -------------------------------------------------------------------
/*     LOCATIONS OF THE LEN PIXELS IN THE SCAN FROM ERTLOC            
/* -------------------------------------------------------------------
/*       lon  ...ARRAY OF LONGITUDES FROM ERTLOC                      
/*                                                                    
/*       lat  ...ARRAY OF LATTITUDES FROME ERTLOC                      
/*                                                                     
/*       PHI.....LOCATION OF A PARTICULAR POINT IN THE VECTOR PHIV     
/*               (GIVEN IN DEGREES LONGITUDE)   [-180,+180]           
/*       RLAMDA..LOCATION OF A PARTICULAR POINT IN THE VECTOR RLAMV    
/*               (GIVEN IN DEGREES LATITUDE)    [-90,+90]             
/*
/* $Log: zenith.c,v $
 * Revision 1.1  1995/02/28  17:12:54  rheil
 * Initial revision
 *
/*  12/15/94  - Code Origination, Robert M. Heil (Bob)
/*
/******************************************************************/
/* UNIX INCLUDE FILES */
#include <stdio.h>
 
/* CREATE_1B INCLUDE FILES */
#include "macros.h"
#include "types.h"
#include "hrpt.h"

/* GLOBAL VARIABLES */
static char zenith_c[] = "$Header: /projects/level1b/create.1bl/C/RCS/zenith.c,v 1.1 1995/02/28 17:12:54 rheil Exp $";

/* Builtin functions */
flt8 sin(), cos(), acos();
 
void zenith(lat,lon,zen,time,date,oldtime,olddate)
flt8 *lat,*lon,*zen;
int4 time, date, oldtime, olddate;
{
    int4 i;
    flt8 pi = 3.141593;
    flt8 dr = 57.29578;
    flt8 ftime, hourang, delmax, delta, y;

    /* COMPUTE THE ZENITH ANGLE FOR EACH OF THE LEN PIXELS SEPARATELY */
    for (i=0;i<LEN;i++)
    {
        /* Convert Gmt To Local Time: */
        ftime = (time/1000.0) + 240.0 * lon[i];  /* 240.00 = Sec/deg */
        if(ftime > 86400.0)   ftime -= 86400.0;
        if(ftime < 0.0)       ftime += 86400.0;
        if(ftime == 86400.0)  ftime = 0.0;

        /* DETERMINE THE HOUR ANGLE (H): */
        /* The Number Of Degrees Of Longitude That The Pixel Is Away */
        /* From The Meridian Directly Under The Sun. */
        /* 12.0 Hrs = 43200 Seconds   15 Deg/hr = 0.004166667 Deg/sec */
        hourang = 0.004166667 * (ftime - 43200.0);
        hourang /= dr;

        /* Compute Delta:
        /* The Solar Inclination Angle (Or Solar Declination) */
        /* Assume:  This Function Of Time Behaves As A Sine Wave (Kuo-nan */
        /* Liou, P. 47) With Delta=0 Deg. On Vernal And Autumnal Equinoxes... */
        /* Delta = 23.45 Deg. = Delmax On Summer Solstice... */
        /* Delta = -23.45 Deg = Delmin On Winter Solstice. */
        delmax=23.45;

        /* Begin Annual Calendar For Sine Wave On Vernal Equinox... */
        /* Delta = Zero On Date = 80....set New Date (D) Equal To Zero On The */
        /* Vernal Equinox. */
        if(date < 80)
            delta = delmax * sin(2*pi*(date + 285.0)/365);
        else
            delta = delmax * sin(2*pi*(date - 80.0)/365);
        delta /= dr;

        /* Compute The Solar Zenith Angle (Theta) From The Hour Angle (H), */
        /* Solar Declination (Delta), And Latitude (Rlamda). */
        y = sin(lat[i]/dr)*sin(delta) + cos(lat[i]/dr)*cos(delta)*cos(hourang);
        zen[i] = dr*acos(y);
        if (zen[i] > 128.0) zen[i] = 128.0;
    }
}
